@@ -1,7 +1,7 @@
 <?php
 /*
  +-------------------------------------------------------------------------+
- | Copyright (C) 2004-2020 The Cacti Group                                 |
+ | Copyright (C) 2004-2021 The Cacti Group                                 |
  |                                                                         |
  | This program is free software; you can redistribute it and/or           |
  | modify it under the terms of the GNU General Public License             |
@@ -85,18 +85,18 @@ function install_create_csrf_secret($file) {
 function install_test_local_database_connection() {
 	global $database_type, $database_hostname, $database_username, $database_password, $database_default, $database_type, $database_port, $database_retries, $database_ssl, $database_ssl_key, $database_ssl_cert, $database_ssl_ca;
 
-	if (!isset($database_ssl)) $rdatabase_ssl = false;
-	if (!isset($database_ssl_key)) $rdatabase_ssl_key = false;
-	if (!isset($database_ssl_cert)) $rdatabase_ssl_cert = false;
-	if (!isset($database_ssl_ca)) $rdatabase_ssl_ca = false;
+	if (!isset($database_ssl)) $database_ssl = false;
+	if (!isset($database_ssl_key)) $database_ssl_key = false;
+	if (!isset($database_ssl_cert)) $database_ssl_cert = false;
+	if (!isset($database_ssl_ca)) $database_ssl_ca = false;
 
 	$connection = db_connect_real($database_hostname, $database_username, $database_password, $database_default, $database_type, $database_port, $database_retries, $database_ssl, $database_ssl_key, $database_ssl_cert, $database_ssl_ca);
 
 	if (is_object($connection)) {
 		db_close($connection);
-		print json_encode(array('status' => 'true'));
+		return json_encode(array('status' => 'true'));
 	} else {
-		print json_encode(array('status' => 'false'));
+		return json_encode(array('status' => 'false'));
 	}
 }
 
@@ -112,9 +112,9 @@ function install_test_remote_database_connection() {
 
 	if (is_object($connection)) {
 		db_close($connection);
-		print json_encode(array('status' => 'true'));
+		return json_encode(array('status' => 'true'));
 	} else {
-		print json_encode(array('status' => 'false'));
+		return json_encode(array('status' => 'false'));
 	}
 }
 
@@ -609,10 +609,21 @@ function install_file_paths() {
 	/* PHP Binary Path */
 	$input['path_php_binary'] = install_tool_path('php_binary',
 		array(
-			'unix'  => '/usr/bin/php',
+			'unix'  => '/bin/php',
 			'win32' => 'c:/php/php.exe'
 		));
 
+	// Workaround to support xampp
+	if ($config['cacti_server_os'] == 'win32') {
+		$paths = array('c:/php/php.exe', 'd:/php/php.exe', 'c:/xampp/php/php.exe', 'd:/xampp/php/php.exe');
+		foreach($paths as $path) {
+			if (file_exists($path)) {
+				$input['path_php_binary']['default'] = $path;
+				break;
+			}
+		}
+	}
+
 	/* RRDtool Binary Path */
 	$input['path_rrdtool'] = install_tool_path('rrdtool',
 		array(
@@ -715,16 +726,18 @@ function install_file_paths() {
 
 function remote_update_config_file() {
 	global $config, $rdatabase_type, $rdatabase_hostname, $rdatabase_username,
-		$rdatabase_password, $rdatabase_default, $rdatabase_type, $rdatabase_port, $rdatabase_ssl;
+		$rdatabase_password, $rdatabase_default, $rdatabase_type, $rdatabase_port, 
+		$rdatabase_ssl, $rdatabase_ssl_key, $rdatabase_ssl_cert, $rdatabase_ssl_ca;
 
 	global $database_type, $database_hostname, $database_username,
-		$database_password, $database_default, $database_type, $database_port, $database_ssl;
+		$database_password, $database_default, $database_type, $database_port, 
+		$database_ssl, $database_ssl_key, $database_ssl_cert, $database_ssl_ca;
 
 	$failure     = '';
 	$newfile     = array();
 	$config_file = $config['base_path'] . '/include/config.php';
 
-	$connection = db_connect_real($rdatabase_hostname, $rdatabase_username, $rdatabase_password, $rdatabase_default, $rdatabase_type, $rdatabase_port, $rdatabase_ssl);
+	$connection = db_connect_real($rdatabase_hostname, $rdatabase_username, $rdatabase_password, $rdatabase_default, $rdatabase_type, $rdatabase_port, $rdatabase_ssl, $rdatabase_ssl_key, $rdatabase_ssl_cert, $rdatabase_ssl_ca);
 
 	if (is_object($connection)) {
 		if (function_exists('gethostname')) {
@@ -748,6 +761,9 @@ function remote_update_config_file() {
 			$save['dbpass']    = $database_password;
 			$save['dbport']    = $database_port;
 			$save['dbssl']     = $database_ssl ? 'on' : '';
+			$save['dbsslkey']  = $database_ssl_key;
+			$save['dbsslcert'] = $database_ssl_cert;
+			$save['dbsslca']   = $database_ssl_ca;
 
 			$poller_id = sql_save($save, 'poller', 'id', true, $connection);
 		}
